"""
Script to create data used for regression testing.

"""

import numpy as np
from numpy import random
import h5py

import bitshuffle
from bitshuffle import h5
from h5py import h5z

BLOCK_SIZE = 64  # Smallish such that datasets have many blocks but are small.
COMP_LVL = 10  # ZSTD compression level
FILTER_PIPELINE = [h5.H5FILTER]
FILTER_OPTS = [
    [(BLOCK_SIZE, h5.H5_COMPRESS_LZ4)],
    [(BLOCK_SIZE, h5.H5_COMPRESS_ZSTD, COMP_LVL)],
]

OUT_FILE = "tests/data/regression_%s.h5" % bitshuffle.__version__

DTYPES = ["a1", "a2", "a3", "a4", "a6", "a8", "a10"]

f = h5py.File(OUT_FILE, "w")
g_orig = f.create_group("origional")
g_comp_lz4 = f.create_group("compressed")
g_comp_zstd = f.create_group("compressed_zstd")

for dtype in DTYPES:
    for rep in ["a", "b", "c"]:
        dset_name = "%s_%s" % (dtype, rep)
        dtype = np.dtype(dtype)
        n_elem = 3 * BLOCK_SIZE + random.randint(0, BLOCK_SIZE)
        shape = (n_elem,)
        chunks = shape
        data = random.randint(0, 255, n_elem * dtype.itemsize)
        data = data.astype(np.uint8).view(dtype)

        g_orig.create_dataset(dset_name, data=data)

        # Create LZ4 compressed data
        h5.create_dataset(
            g_comp_lz4,
            bytes(dset_name, "utf-8"),
            shape,
            dtype,
            chunks=chunks,
            filter_pipeline=FILTER_PIPELINE,
            filter_flags=(h5z.FLAG_MANDATORY,),
            filter_opts=FILTER_OPTS[0],
        )
        g_comp_lz4[dset_name][:] = data

        # Create ZSTD compressed data
        h5.create_dataset(
            g_comp_zstd,
            bytes(dset_name, "utf-8"),
            shape,
            dtype,
            chunks=chunks,
            filter_pipeline=FILTER_PIPELINE,
            filter_flags=(h5z.FLAG_MANDATORY,),
            filter_opts=FILTER_OPTS[1],
        )
        g_comp_zstd[dset_name][:] = data

f.close()



import time
import perf_utils
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient
from libertem_live.detectors.dectris import DectrisAcquisition
from libertem_live.api import LiveContext
from libertem.udf.sumsigudf import SumSigUDF


if __name__ == "__main__":
    ctx = LiveContext()

    aq = DectrisAcquisition(
        api_host='localhost',
        api_port=8910,
        data_host='localhost',
        data_port=9999,
        nav_shape=(256, 256),
        trigger_mode='exte',
        trigger=lambda x: None,
        frames_per_partition=1024,
    ) 
    aq = aq.initialize(ctx.executor)
    # warmup:
    ctx.run_udf(
        dataset=aq,
        udf=SumSigUDF(),
    )
    for i in range(10):
        t0 = time.time()
        ctx.run_udf(
            dataset=aq,
            udf=SumSigUDF(),
        )
        t1 = time.time()
        print(t1-t0)
    ctx.close()
    print("done")



import time
import click
import perf_utils
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient
from libertem_live.detectors.dectris import DectrisAcquisition
from libertem_live.api import LiveContext
from libertem.udf.sumsigudf import SumSigUDF


@click.command()
@click.argument('width', type=int, default=512)
@click.argument('height', type=int, default=512)
@click.option('--perf', type=bool, default=False)
def main(width: int, height: int, perf: bool):

    ctx = LiveContext()

    aq = DectrisAcquisition(
        api_host='localhost',
        api_port=8910,
        data_host='localhost',
        data_port=9999,
        nav_shape=(height, width),
        trigger_mode='exte',
        trigger=lambda x: None,
        frames_per_partition=1024,
    ) 
    aq = aq.initialize(ctx.executor)
    print("warmup")
    ctx.run_udf(
        dataset=aq,
        udf=SumSigUDF(),
    )
    if perf:
        with perf_utils.perf('testudf', output_dir='profiles', sample_frequency='max') as perf_data:
            t0 = time.time()
            ctx.run_udf(
                dataset=aq,
                udf=SumSigUDF(),
            )
            t1 = time.time()
            print(t1-t0)
    else:
        t0 = time.time()
        ctx.run_udf(
            dataset=aq,
            udf=SumSigUDF(),
        )
        t1 = time.time()
        print(t1-t0)
        print(f"{aq.shape.nav.size/(t1-t0)}")
    ctx.close()
    print("done")


if __name__ == "__main__":
    main()



import time
import libertem_dectris

if __name__ == "__main__":
    frames = libertem_dectris.FrameIterator(uri="tcp://127.0.0.1:9999")

    frames.start(series=14)

    for frame in frames:
        time.sleep(1.1)

    frames.close()




"""
Test that data encoded with earlier versions can still be decoded correctly.

"""

from __future__ import absolute_import, division, print_function

import pathlib
import unittest

import numpy as np
import h5py
from bitshuffle import __zstd__

from packaging import version

TEST_DATA_DIR = pathlib.Path(__file__).parent / "data"

OUT_FILE_TEMPLATE = "regression_%s.h5"

VERSIONS = ["0.1.3", "0.4.0"]


class TestAll(unittest.TestCase):
    def test_regression(self):
        for rev in VERSIONS:
            file_name = TEST_DATA_DIR / (OUT_FILE_TEMPLATE % rev)
            f = h5py.File(file_name, "r")
            g_orig = f["original"]
            g_comp = f["compressed"]

            for dset_name in g_comp.keys():
                self.assertTrue(np.all(g_comp[dset_name][:] == g_orig[dset_name][:]))

            # Only run ZSTD comparison on versions >= 0.4.0 and if ZSTD support
            # has been built into bitshuffle
            if version.parse(rev) >= version.parse("0.4.0") and __zstd__:
                g_comp_zstd = f["compressed_zstd"]
                for dset_name in g_comp_zstd.keys():
                    self.assertTrue(
                        np.all(g_comp_zstd[dset_name][:] == g_orig[dset_name][:])
                    )


if __name__ == "__main__":
    unittest.main()



from __future__ import absolute_import, division, print_function, unicode_literals

import unittest
import os
import glob

import numpy as np
import h5py
import pytest
from h5py import h5z

from bitshuffle import h5, __zstd__


os.environ["HDF5_PLUGIN_PATH"] = ""


class TestFilter(unittest.TestCase):
    def test_filter(self):
        shape = (32 * 1024 + 783,)
        chunks = (4 * 1024 + 23,)
        dtype = np.int64
        data = np.arange(shape[0])
        fname = "tmp_test_filters.h5"
        f = h5py.File(fname, "w")
        h5.create_dataset(
            f,
            b"range",
            shape,
            dtype,
            chunks,
            filter_pipeline=(32008, 32000),
            filter_flags=(h5z.FLAG_MANDATORY, h5z.FLAG_MANDATORY),
            filter_opts=None,
        )
        f["range"][:] = data

        f.close()

        f = h5py.File(fname, "r")
        d = f["range"][:]
        self.assertTrue(np.all(d == data))
        f.close()

    def test_with_block_size(self):
        shape = (128 * 1024 + 783,)
        chunks = (4 * 1024 + 23,)
        dtype = np.int64
        data = np.arange(shape[0])
        fname = "tmp_test_filters.h5"
        f = h5py.File(fname, "w")
        h5.create_dataset(
            f,
            b"range",
            shape,
            dtype,
            chunks,
            filter_pipeline=(32008, 32000),
            filter_flags=(h5z.FLAG_MANDATORY, h5z.FLAG_MANDATORY),
            filter_opts=((680,), ()),
        )
        f["range"][:] = data

        f.close()
        # os.system('h5dump -H -p tmp_test_filters.h5')

        f = h5py.File(fname, "r")
        d = f["range"][:]
        self.assertTrue(np.all(d == data))
        f.close()

    def test_with_lz4_compression(self):
        shape = (128 * 1024 + 783,)
        chunks = (4 * 1024 + 23,)
        dtype = np.int64
        data = np.arange(shape[0])
        fname = "tmp_test_filters.h5"
        f = h5py.File(fname, "w")
        h5.create_dataset(
            f,
            b"range",
            shape,
            dtype,
            chunks,
            filter_pipeline=(32008,),
            filter_flags=(h5z.FLAG_MANDATORY,),
            filter_opts=((0, h5.H5_COMPRESS_LZ4),),
        )
        f["range"][:] = data

        f.close()
        # os.system('h5dump -H -p tmp_test_filters.h5')

        f = h5py.File(fname, "r")
        d = f["range"][:]
        self.assertTrue(np.all(d == data))
        f.close()

    @pytest.mark.skipif(
        __zstd__ is False,
        reason="Bitshuffle has not been built with ZSTD support.",
    )
    def test_with_zstd_compression(self):
        shape = (128 * 1024 + 783,)
        chunks = (4 * 1024 + 23,)
        compression_lvl = 10
        dtype = np.int64
        data = np.arange(shape[0])
        fname = "tmp_test_filters.h5"
        f = h5py.File(fname, "w")
        h5.create_dataset(
            f,
            b"range",
            shape,
            dtype,
            chunks,
            filter_pipeline=(32008,),
            filter_flags=(h5z.FLAG_MANDATORY,),
            filter_opts=((0, h5.H5_COMPRESS_ZSTD, compression_lvl),),
        )
        f["range"][:] = data

        f.close()
        # os.system('h5dump -H -p tmp_test_filters.h5')

        f = h5py.File(fname, "r")
        d = f["range"][:]
        self.assertTrue(np.all(d == data))
        f.close()

    def tearDown(self):
        files = glob.glob("tmp_test_*")
        for f in files:
            os.remove(f)


if __name__ == "__main__":
    unittest.main()



import libertem_dectris

if __name__ == "__main__":
    frames = libertem_dectris.FrameChunkedIterator(uri="tcp://127.0.0.1:9999")
    frames.start()

    frame_stack = frames.get_next_stack(max_size=32)
    serialized = frame_stack.serialize()
    loaded = libertem_dectris.FrameStack.deserialize(serialized)

    print(serialized)
    print(len(loaded))
    print(loaded[0])
    print(loaded[0].get_pixel_type())
    print(loaded[0].get_encoding())
    print(loaded[0].get_shape())
